#-----------------------------------------------------------
# Test of the python interpreter and use of animation in
# xcircuit. Execute this script using menu option
# "File/Execute Script", if Python has been compiled in.
#-----------------------------------------------------------

from math import pi,sin,cos

def move(h1, x, y):
   d = {"position": (x, y)}
   setattr(h1, d) 

def newarc(x, y, r):
   h1=newelement("Arc")
   d = {"radius": r, "minor axis": r, "position": (x, y)}
   setattr(h1, d)
   return h1

x = y = 0
x2 = y2 = 0
bigrx = 400
bigry = 200
nsteps = 200
step = 2 * pi / (nsteps - 1) 

set("grid","off")
set("axis","off")
set("snap","off")

h1 = newarc(x, y, 100)
h2 = newarc(x2, y2, 85)

pause(0.5)
for i in range(0,nsteps):
   x2 = x
   y2 = y
   x = int(round(bigrx * sin(i * step)))
   y = int(round(bigry * cos(i * step)))
   move(h1, x, y)
   move(h2, x2, y2)
#  pause(0.01)
   refresh();

set("grid","on")
set("axis","on")
set("snap","on")

#-----------------------------------------------------------



# XCircuit Python startup script (xcstartup.py)
# Does the work previously handled by "builtins.lps".  Commands
# "loadlibrary" and "loadfontencoding" should no longer be used.
#
#  Written by Tim Edwards 1/19/01 (tim@bach.ece.jhu.edu)
#  The Johns Hopkins University

font("times_roman.xfe")
font("times_romaniso.xfe")
font("helvetica.xfe")
font("helveticaiso.xfe")
font("courier.xfe")
font("courieriso.xfe")
font("symbol.xfe")

# Alternate font encodings:  Uncomment these if you want them
# loaded by default
#
# font("times_romaniso2.xfe")
# font("courieriso2.xfe")
# font("helveticaiso2.xfe")
# font("times_romaniso5.xfe")
# font("courieriso5.xfe")
# font("helveticaiso5.xfe")

# First library page
library("generic.lps", 1)
library("analog.lps", 1)
library("avlsi.lps", 1)
library("digital.lps", 1)

# Second library page
library ("analoglib2.lps", 2)

# Third library page
library ("ic_templates.lps", 3)

# Fourth library page
library ("quadparts.lps", 4)




#-----------------------------------------#
# xcircuitrc file for editing sheet music #
# (python version)			  #
# Rename this file to .xcircuitrc	  #
#-----------------------------------------#

# 1) Don't load any of the default libraries.
#    Instead, load only "musiclib".
override("library")
library("/usr/local/lib/xcircuit-3.6/musiclib")

# 2) Load font Times-Roman and set the default font to Times-RomanISO
font("Times-Roman")
set("font", "Times-RomanISO")

# 3) Set various parameters for music editing
set("boxedit", "rhomboid-y")
set("colorscheme", "inverse")
set("xschema", "off")

# 4) Set Backspace = Delete
unbind("BackSpace", "Text Left")
bind("BackSpace", "Text Delete")



# gettext.py
#-----------------------------------------------------------
# Python script which creates a function "gettext(filename)"
# that reads an ASCII file and turns the contents into a
# text label.
# Execute this script using menu option "File/Execute Script",
# if Python has been compiled in.
#
# Also:  defines a function which can be called with a key
# macro (gets the filename via popup prompt) and binds it
# to key "^G" (ctrl-"g"), and defines a button named "Get Text"
# under the "Edit" menu which does the same.
#-----------------------------------------------------------

def newlabel(x, y, l):
   h1=newelement("Label")
   d = {"scale": 1.0, "rotation": 0, "justify": 0, "pin": 0,
	"position": (x, y), "string": l}
   setattr(h1, d)
   return h1

def gettext(f):
   try:
      fi = open(f, 'r')
   except IOError:
      return
   else:
      T = getcursor();
      D = {'Font': 'Times-Roman'}
      S2 = [D]
      S = fi.readlines()
      for X in S:
         Y = X[0:len(X)-1]
	 D = {'Text': Y}
         S2.append(D)
         S2.append('Return')
      h1 = newlabel(T[0], T[1], S2)
      return h1

def prompttext():
   filepopup('Enter filename to import:', 'gettext')

bind('Control_G', 'prompttext')
newbutton('Edit', 'Get Text (^G)', 'prompttext')

#-----------------------------------------------------------



# spice.py
#-----------------------------------------------------------------------
# Python script which writes a SPICE-format netlist.
# Replaces the code formerly in "netlist.c" (deprecated).
# Python scripting is now the preferred method for handling
# netlist output formats.
#-----------------------------------------------------------------------

import string

#-----------------------------------------------------------------------
# Parse the output string, substituting for index, ports, parameters,
# and automatic numbering using the '?' parameter.
#-----------------------------------------------------------------------

def parsedev(lstr, portdict, devdict, classidx, paramdict):
   rstr = lstr
   ltext = ''
   classname = rstr[0]		# single-character SPICE element type
   try:
      index = classidx[classname]
   except KeyError:
      index = 1
      classidx[classname] = 1

   if string.find(rstr, '%i'):
      rstr = string.replace(rstr, '%i', str(index))
      classidx[classname] = classidx[classname] + 1

   for pt in portdict.keys():
      ptstr = '%p' + pt
      rstr = string.replace(rstr, ptstr, portdict[pt])
      ptstr = '%p' + '"' + pt + '"'
      rstr = string.replace(rstr, ptstr, portdict[pt])

   for vt in paramdict.keys():
      vtvalue = paramdict[vt]
      if vtvalue == '?':
	 vtvalue = str(index);
	 classidx[classname] = classidx[classname] + 1
     
      vtstr = '%v' + vt
      rstr = string.replace(rstr, vtstr, vtvalue)
      vtstr = '%v' + '"' + vt + '"'
      rstr = string.replace(rstr, vtstr, vtvalue)

   # Simple replacements (%%, %r, %t) (all occurrences)
   #...................................................
   if string.find(rstr, '%%'):
      rstr = string.replace(rstr, '%%', '%')
   if string.find(rstr, '%n'):
      rstr = string.replace(rstr, '%n', devdict['name'])
   if string.find(rstr, '%r'):
      rstr = string.replace(rstr, '%r', '\n')
   if string.find(rstr, '%t'):
      rstr = string.replace(rstr, '%t', '\t')

   return rstr

#-----------------------------------------------------------------------
# Generate a dictionary of port substitutions and create the
# device string for output
#-----------------------------------------------------------------------

def devwrite(calldict, devdict, classidx):
   ltext = ''

   # Resolve network connections to ports and store as a dictionary
   #...............................................................
   portdict = {}
   ports_from = calldict['ports']
   ports_to = devdict['ports']
   for i in range(0,len(ports_from)):
      portdict[textprint(ports_to[i], params)] = textprint(ports_from[i], [])

   # Resolve instanced vs. default parameters and store as a dictionary
   # (only for use with the "%v" statement in info labels)
   #...............................................................
   paramdict = {}
   try:
      def_params = devdict['parameters']
   except KeyError:
      def_params = []

   try:
      params = calldict['parameters']
   except KeyError:
      params = []

   params += def_params[len(params):]

   for i in range(0,len(params)):
      paramdict[textprint(def_params[i], [])] = textprint(params[i], [])

   # For each "spice:" info label, parse the info label
   #...............................................................
   w = devdict['devices']
   for y in w:
      for u in y:
	 lstr = select(textprint(u, params), 'spice')
	 if lstr <> '':
	    ltext += parsedev(lstr, portdict, devdict, classidx, paramdict)
	    ltext += '\n'
   return ltext
	 
#-----------------------------------------------------------------------
# Check if a device is "fundamental";  that is, has a "devices" dictionary
# with at least one entry with "spice:" for SPICE output.
#-----------------------------------------------------------------------

def isfundamental(devdict):
   try:
      w = devdict['devices']
   except KeyError:
      return 0
   else:
      for y in w:
	 for u in y:
	    lstr = select(textprint(u, []), 'spice')
	    if lstr <> '':
	       return 1
   return 0

#-----------------------------------------------------------------------
# Select the device string corresponding to the given prefix.
# Return the body of the string, or an empty string if the prefix
# doesn't match.
#-----------------------------------------------------------------------

def select(sstr, prefix):
   ltext = ''
   if sstr.startswith(prefix):
      ltext += sstr[len(prefix) + 1:]
   return ltext
	 
#-----------------------------------------------------------------------
# Generate an ASCII string from an xcircuit string (list).  Translate
# overlines to "!" and Greek "mu" (either from Symbol font or from the
# ISO-Latin1 encoding of any standard font) to "u".
# String parameter substitution is performed if "params" is a non-empty
# list and the dictionary key "Parameter" is encountered in the string
# list "slist".
#-----------------------------------------------------------------------

def textprint(slist, params):
   ltext = ''
   is_symbol = 0
   is_iso = 0
   for x in slist:
      try:
	f = x.keys()[0]
      except AttributeError:		# must be a string
	if x == 'Return':
	   ltext += '\n'
	elif x == 'Underline':
	   ltext += '_'
	elif x == 'Overline':
	   ltext += '!'
      else:			# is a dictionary; will have only one key
	if f == 'Font':
	   lfont = x[x.keys()[0]]
	   if lfont.startswith('Symbol'):
	      is_symbol = 1
	   else:
	      is_symbol = 0
	   if lfont.endswith('ISO'):
	      is_iso = 1
	   else:
	      is_iso = 0
	elif f == 'Parameter':
	   try:
	      plist = params[x[x.keys()[0]]]
	   except IndexError:
	      ltext += ' --unknown parameter %d-- ' % x[x.keys()[0]]
	   else:
	      ltext += textprint(plist, [])
	   continue;
	else:				# text:  SPICE translates "mu" to "u"
	   for y in x[x.keys()[0]]:
	      if is_symbol:
		 if y == 'f':
		    ltext += 'phi'
		 elif y == 'm':
		    ltext += 'u'
		 else:
		    ltext += y
	      else:
		 if ord(y) == 181:
		    ltext += 'u'
		 elif ord(y) > 127:
		    ltext += '/' + str(ord(y))
		 else:
		    ltext += y
   return ltext
	
#-----------------------------------------------------------------------
# Write netlist to the output
#-----------------------------------------------------------------------

def writespice():
   subidx=1		# subcircuit numbering
   devdict={}		# dictionary of low-level devices
   classidx = {}	# indices of SPICE devices (other than subcircuits)
   p=netlist()		# Generate the netlist
   g=p['globals']	# Get the list of global networks
   c=p['circuit']	# Get the list of (sub)circuits
   l=len(c)
   top=c[l-1]		# Top-level circuit
   topname=top['name']	# Name of the top-level circuit
   topname += '.spc'	# Filename to write to
   try:
      outfile=open(topname, 'w')
   except IOError:
      return

   # Print header line
   #...............................................................

   outfile.write('*SPICE circuit "' + topname + '"')
   outfile.write(' from XCircuit v' + str(xc_version))
   outfile.write(' (Python script "spice.py")\n')

   # Print global variables.  This is for hspice and should be
   # commented out for spice3.
   #...............................................................

   for x in g:	# 'globals' is a list of strings
      outfile.write('.GLOBAL ' + textprint(x, []) + '\n')
   outfile.write('\n')

   # Print the (sub)circuits.  Top level circuit does not get a "subckt"
   # entry.
   #...............................................................

   for x in c:	# 'circuits' is a list of dictionaries
      is_device = 0
      try:
	 w = x['ports']		# write circuit name and ports, if any
      except KeyError:
	 is_top = 1
      else:
	 is_top = 0
	 if isfundamental(x) == 0:
            outfile.write('.subckt ' + x['name'])
            for y in w:
	       outfile.write(' ' + textprint(y, []))
            outfile.write('\n')
	 else:
	    is_device = 1
	    devdict[x['name']] = x

      try:
	 v = x['calls']		# calls to subcircuits in xcircuit
      except KeyError:
	 pass
      else:
         for y in v:
	    try:
	       d = devdict[y['name']]
	    except KeyError:		# A SPICE subcircuit instance
	       outfile.write('X' + str(subidx))
	       subidx+=1
	       for z in y['ports']:
	          outfile.write(' ' + textprint(z, []))
	       outfile.write(' ' + y['name'] + '\n') 
	    else:			# A SPICE circuit element
	       nstr = devwrite(y, d, classidx);
	       outfile.write(nstr)

      if is_top:
	 outfile.write('.end\n')
      elif is_device == 0:
	 outfile.write('.ends\n\n')

#-----------------------------------------------------------------------
# Key binding and menu button for the spice netlist output
# bind('Alt_S', 'writespice')
#-----------------------------------------------------------------------

newbutton('Netlist', 'Write Spice', 'writespice')

#-----------------------------------------------------------------------



# pagebbox.py
#-----------------------------------------------------------
# Python script which creates a function "pagebbox(width, height)"
# that generates a bounding box of the specified width and height.
#
# Execute this script using menu option "File/Execute Script",
# if Python has been compiled in.
#
# Add key 'B' for popuppromt asking for page size.  Add button to
# "Edit" menu for same.  Add Ctrl-B and Ctrl-Shift-B for standard
# 8 1/2 x 11 and 11 x 17 page bounding boxes.
#-----------------------------------------------------------

def newbox(lx, ly, ux, uy):
   h1=newelement("Polygon")
   plist = [(lx, ly), (ux, ly), (ux, uy), (lx, uy)]
   d = {"points": plist}
   setattr(h1, d)
   return h1

def pagebbox(w, h):
   h1 = newbox(0, 0, w, h)
   d = {"style": 512, "color": (255, 165, 0)}
   setattr(h1, d)
   return h1

def intbbox(s):
   slist = s.split()
   slen = len(slist)
   if slen > 1:
      w = eval(slist[0])
   else:
      w = 8.5

   if slen == 3:
      h = eval(slist[2])
   elif slen == 2:
      h = eval(slist[1])
   else:
      h = 11

   # note to self:  should look at coord style, set default for A4 size
   # if style is CM, not INCHES.

   p1 = getpage()
   oscale = p1['output scale']

   inchsc = 192 
   h1 = pagebbox(w * oscale * inchsc, h * oscale * inchsc)
   return h1

def promptbbox():
   popupprompt('Enter width x height:', 'intbbox')

def stdbbox():
   intbbox("8.5 x 11")

def bigbbox():
   intbbox("11 x 17")

bind('B', 'promptbbox')
bind('Control_b', 'stdbbox')
bind('Control_Shift_B', 'bigbbox')

newbutton('Edit', 'New BBox (B)', 'promptbbox')
# newtool('BBox', 'promptbbox', '', 'bounding box');

#-----------------------------------------------------------



# The followin Python script replaces the zoom in/out operators
# with a pan function followed by a zoom.  The range of the zoom
# can be adjusted by this method, too.

def panzoomin():
  T=getcursor()
  pan(T)
  zoom(1.5)

def panzoomout():
  T=getcursor()
  pan(T)
  zoom(0.66667)

unbind("Z", "Zoom In")
unbind("z", "Zoom Out")

bind("Z", "panzoomin")
bind("z", "panzoomout")



# flatspice.py
#-----------------------------------------------------------
# Python script which writes a SPICE-format netlist.
# Replaces the code formerly in "netlist.c" (deprecated).
# Python scripting is now the preferred method for handling
# netlist output formats.
#-----------------------------------------------------------

# Select the device string corresponding to the given prefix.
# Return the body of the string, or an empty string if the prefix doesn't match.

def select(sstr, prefix):
   ltext = ''
   if sstr.startswith(prefix):
      ltext += sstr[len(prefix) + 1:]
   return ltext
	 
# Generate an ASCII string from an xcircuit string (list)

def textprint(slist, params):
   ltext = ''
   is_symbol = 0
   is_iso = 0
   for x in slist:
      try:
	f = x.keys()[0]
      except AttributeError:		# must be a string
	if x == 'Return':
	   ltext += '\n'
	elif x == 'Underline':
	   ltext += '_'
	elif x == 'Overline':
	   ltext += '!'
      else:			# is a dictionary; will have only one key
	if f == 'Font':
	   lfont = x[x.keys()[0]]
	   if lfont.startswith('Symbol'):
	      is_symbol = 1
	   else:
	      is_symbol = 0
	   if lfont.endswith('ISO'):
	      is_iso = 1
	   else:
	      is_iso = 0
	elif f == 'Parameter':
	   ltext += textprint(params[x[x.keys()[0]]], [])
	else:				# text:  SPICE translates "mu" to "u"
	   for y in x[x.keys()[0]]:
	      if is_symbol:
		 if y == 'f':
		    ltext += 'phi'
		 elif y == 'm':
		    ltext += 'u'
		 else:
		    ltext += y
	      else:
		 if ord(y) == 181:
		    ltext += 'u'
		 elif ord(y) > 127:
		    ltext += '/' + str(ord(y))
		 else:
		    ltext += y
   return ltext
	

# Flatten the netlist and write to the output

def recurseflat(outfile, ckt, clist):
   try:
      v = ckt['calls']		# calls to subcircuits
   except KeyError:		# A bottom-level circuit element
      pass
   else:
      for y in v:
	 for z in clist:
	    if z['name'] == y['name']:
	       # copy the object and substitute net names into subcircuit ports
	       lobj = z
	       lobj['ports'] = y['ports']
	       recurseflat(outfile, lobj, clist)
	       break;
   try:
      w = ckt['devices']
   except KeyError:
      pass
   else:
      for y in w:
	 for u in y:
	    lstr = select(textprint(u, []), 'spice')
	    if lstr <> '':
	       outfile.write('device: ' + lstr + '\n')

# Top of the flattened-circuit writing routine

def writespiceflat():
   p=netlist()
   g=p['globals']
   c=p['circuit']
   l=len(c)
   top=c[l-1]
   topname=top['name']
   topname += '.spc'
   try:
      outfile=open(topname, 'w')
   except IOError:
      return

   # print header line

   outfile.write('*SPICE flattened circuit "' + topname + '"')
   outfile.write(' from XCircuit v' + str(xc_version))
   outfile.write(' (Python script "flatspice.py")\n')

   # print global variables

   for x in g:	# 'globals' is a list of strings
      outfile.write('.GLOBAL ' + textprint(x, []) + '\n')
   outfile.write('\n')

   recurseflat(outfile, top, c)
   outfile.write('.end\n')
   outfile.close()

# Key binding and menu button for the spice netlist output
# bind('Alt_F', 'writespiceflat')
newbutton('Netlist', 'Write Flattened Spice', 'writespiceflat')




# loadlgf.py
#-----------------------------------------------------------
# Python script which creates a function "loadlgf" that
# replaces the code formerly in "formats.c" (deprecated).
# Python scripting is now the preferred method for handling
# alternate file formats.
#-----------------------------------------------------------

def loadlgf(f):
   try:
      fi = open(f, 'r')
   except IOError:
      return
   else:

      # check magic cookie to see if it's a real LGF file

      S = fi.readline()
      if (S <> '-5\n'):
	 return

      S = fi.readline()
      if (S <> 'f s\n'):
	 return

      # Now go load the LGF library (required)
      # '-1' loads at the end of the current library pages

      library('lgf.lps', -1)

      # clear the page

      reset()

      # read in the file

      S = fi.readlines()
      for X in S:
	 if (S[0] == '#'):
	 elif (S[0] == 'n'):
	 elif (S[0] == 's'):
	 elif (S[0] == 'l'):
	 elif (S[0] == 'w'):
	 elif (S[0] == 'p'):
	 elif (S[0] == 'b'):
	 elif (S[0] == 'g'):
	 elif (S[0] == 'h'):
	 elif (S[0] == '.'):
	 else:

      h1 = getpage();
      return h1

def promptlgf():
   filepopup('Enter filename to load:', 'loadlgf')

bind('Control_l', 'promptlgf')
newbutton('Edit', 'Load LGF File (^L)', 'promptlgf')

#-----------------------------------------------------------
